.. _Puppet: http://puppetlabs.com/puppet/puppet-open-source
.. _Salt: http://saltstack.com/community
.. _Ansible: http://www.ansibleworks.com
.. _Hiera: http://projects.puppetlabs.com/projects/hiera
.. _Artistic Licence 2.0: http://opensource.org/licenses/Artistic-2.0
.. _Jinja2: http://jinja.pocoo.org
